#!/usr/bin/env node

import { input, select } from '@inquirer/prompts';
import { execSync } from 'child_process';
import pkg from 'fs-extra';
import { resolve } from 'path';
import { getCommand, isPackageManager } from './utils.js';
import { BASIC_INTEGRATION } from './constants.js';

const { ensureDirSync, writeFileSync, existsSync } = pkg;

const directory = await input({
  message: 'Where do you want to create the integration?',
  default: '.',
});

const destination = resolve(process.cwd(), directory);

const packageName = await input({
  message: 'What is the slug of the integration? (e.g. my-integration)',
  default: destination.split('/').pop(),
  validate: (input) => {
    // should only be lowercase letters, numbers, and dashes
    if (!/^[a-z0-9-]+$/.test(input)) {
      return 'Package name can only contain lowercase letters, numbers, and dashes';
    }
    return true;
  },
});

const packagePrefix = await input({
  message: 'What is the package prefix of the integration? (e.g. @netlify)',
  validate: (input) => {
    if (input === '') {
      return true;
    }

    // should only be lowercase letters, numbers, and dashes or optionally start with @
    if (!/^@?[a-z0-9-]+$/.test(input)) {
      return 'Package prefix can only contain lowercase letters, numbers, and dashes';
    }

    return true;
  },
});

const packageManager = await select<string>({
  message: 'Which package manager do you want to use?',
  choices: ['npm', 'yarn', 'pnpm'].map((choice) => ({
    title: choice,
    value: choice,
  })),
});

if (!isPackageManager(packageManager)) {
  throw new Error('Invalid package manager');
}

const integrationPackageJson = {
  name: packageName,
  version: '0.0.1',
  main: 'src/index.ts',
  type: 'module',
  scripts: {
    'build:buildtime': 'ntli build -b',
    preview: 'ntli preview',
  },
  dependencies: {
    '@netlify/sdk': 'latest',
    typescript: 'latest',
  },
};

const integrationTsConfig = {
  compilerOptions: {
    target:
      'ES2022' /* Set the JavaScript language version for emitted JavaScript and include compatible library declarations. */,
    module: 'ES2022' /* Specify what module code is generated. */,
    moduleResolution: 'bundler',
    rootDir: '.',
  },
  exclude: ['node_modules', 'dist'],
};

ensureDirSync(destination);

if (existsSync(resolve(destination, 'package.json'))) {
  throw new Error(
    'There is seemingly already a package.json in this directory. Please remove it and try again.',
  );
}

writeFileSync(
  resolve(destination, 'package.json'),
  JSON.stringify(integrationPackageJson, null, 2),
);
writeFileSync(
  resolve(destination, 'tsconfig.json'),
  JSON.stringify(integrationTsConfig, null, 2),
);

console.log('Installing dependencies...');
execSync(getCommand('install', packageManager, []), {
  cwd: destination,
});

console.log('Initialising integration...');

execSync(
  getCommand('run', packageManager, [
    'ntli',
    'init',
    '-s',
    packageName,
    ...(packagePrefix === '' ? [] : ['-p', packagePrefix]),
  ]),
  {
    cwd: destination,
  },
);

ensureDirSync(resolve(destination, 'src'));
writeFileSync(resolve(destination, 'src/index.ts'), BASIC_INTEGRATION);

console.log('Done! ✅');
